@@ -3,21 +3,26 @@ import logging
 
 from opik.evaluation.models.litellm import warning_filters
 
-from opik_optimizer.evolutionary_optimizer.evolutionary_optimizer import (
-    EvolutionaryOptimizer,
-)
-
 from . import datasets
 from .optimizable_agent import OptimizableAgent
-from .optimization_config.chat_prompt import ChatPrompt
+from .api_objects.chat_prompt import ChatPrompt
 from .base_optimizer import BaseOptimizer
-from .few_shot_bayesian_optimizer import FewShotBayesianOptimizer
-from .gepa_optimizer import GepaOptimizer
+from .algorithms import (
+    GepaOptimizer,
+    MetaPromptOptimizer,
+    EvolutionaryOptimizer,
+    HierarchicalReflectiveOptimizer,
+    FewShotBayesianOptimizer,
+    ParameterOptimizer,
+)
 from .logging_config import setup_logging
-from .meta_prompt_optimizer import MetaPromptOptimizer
-from .mipro_optimizer import MiproOptimizer
-from .optimization_config.configs import TaskConfig
 from .optimization_result import OptimizationResult
+from .multi_metric_objective import MultiMetricObjective
+from .algorithms.parameter_optimizer import (
+    ParameterSearchSpace,
+    ParameterSpec,
+    ParameterType,
+)
 
 __version__ = importlib.metadata.version("opik_optimizer")
 
@@ -32,11 +37,15 @@ __all__ = [
     "FewShotBayesianOptimizer",
     "GepaOptimizer",
     "MetaPromptOptimizer",
-    "MiproOptimizer",
     "EvolutionaryOptimizer",
+    "HierarchicalReflectiveOptimizer",
+    "ParameterOptimizer",
     "OptimizationResult",
     "OptimizableAgent",
     "setup_logging",
     "datasets",
-    "TaskConfig",
+    "MultiMetricObjective",
+    "ParameterSearchSpace",
+    "ParameterSpec",
+    "ParameterType",
 ]
