@@ -1,10 +1,14 @@
+import asyncio
 import functools
-import pyrate_limiter
 import time
-import opik.config
+from collections.abc import Awaitable, Callable
+from typing import ParamSpec, TypeVar
 
-from typing import Any
-from collections.abc import Callable
+import opik.config
+import pyrate_limiter
+
+P = ParamSpec("P")
+R = TypeVar("R")
 
 
 class RateLimiter:
@@ -23,13 +27,17 @@ class RateLimiter:
         while not self.limiter.try_acquire(self.bucket_key):
             time.sleep(0.01)
 
+    async def acquire_async(self) -> None:
+        while not self.limiter.try_acquire(self.bucket_key):
+            await asyncio.sleep(0.01)
 
-def rate_limited(limiter: RateLimiter) -> Callable[[Callable], Callable]:
+
+def rate_limited(limiter: RateLimiter) -> Callable[[Callable[P, R]], Callable[P, R]]:
     """Decorator to rate limit a function using the provided limiter"""
 
-    def decorator(func: Callable) -> Callable:
+    def decorator(func: Callable[P, R]) -> Callable[P, R]:
         @functools.wraps(func)
-        def wrapper(*args: Any, **kwargs: Any) -> Any:
+        def wrapper(*args: P.args, **kwargs: P.kwargs) -> R:
             limiter.acquire()
             return func(*args, **kwargs)
 
@@ -38,6 +46,22 @@ def rate_limited(limiter: RateLimiter) -> Callable[[Callable], Callable]:
     return decorator
 
 
+def rate_limited_async(
+    limiter: RateLimiter,
+) -> Callable[[Callable[P, Awaitable[R]]], Callable[P, Awaitable[R]]]:
+    """Decorator to rate limit async functions without blocking the loop"""
+
+    def decorator(func: Callable[P, Awaitable[R]]) -> Callable[P, Awaitable[R]]:
+        @functools.wraps(func)
+        async def wrapper(*args: P.args, **kwargs: P.kwargs) -> R:
+            await limiter.acquire_async()
+            return await func(*args, **kwargs)
+
+        return wrapper
+
+    return decorator
+
+
 def get_rate_limiter_for_current_opik_installation() -> RateLimiter:
     opik_config = opik.config.OpikConfig()
     max_calls_per_second = 10 if opik_config.is_cloud_installation else 50
