@@ -13,17 +13,14 @@ import importlib.metadata
 import litellm
 from opik.rest_api.core import ApiError
 from opik.api_objects import optimization
-from opik import Dataset
+from opik import Dataset, opik_context
 from pydantic import BaseModel
 
-from . import _throttle, optimization_result
-from .cache_config import initialize_cache
-from .optimization_config import chat_prompt, mappers
+from . import optimization_result
+from .api_objects import chat_prompt
 from .optimizable_agent import OptimizableAgent
-from .utils import create_litellm_agent_class, optimization_context
-from . import task_evaluator
-
-_limiter = _throttle.get_rate_limiter_for_current_opik_installation()
+from .utils import create_litellm_agent_class
+from . import task_evaluator, helpers
 
 # Don't use unsupported params:
 litellm.drop_params = True
@@ -56,41 +53,47 @@ class BaseOptimizer(ABC):
         model: str,
         verbose: int = 1,
         seed: int = 42,
-        **model_kwargs: Any,
+        model_parameters: dict[str, Any] | None = None,
+        name: str | None = None,
     ) -> None:
         """
         Base class for optimizers.
 
         Args:
-           model: LiteLLM model name
-           verbose: Controls internal logging/progress bars (0=off, 1=on).
-           seed: Random seed for reproducibility (default: 42)
-           model_kwargs: additional args for model (eg, temperature)
+           model: LiteLLM model name for optimizer's internal reasoning/generation calls
+           verbose: Controls internal logging/progress bars (0=off, 1=on)
+           seed: Random seed for reproducibility
+           model_parameters: Optional dict of LiteLLM parameters for optimizer's internal LLM calls.
+               Common params: temperature, max_tokens, max_completion_tokens, top_p,
+               presence_penalty, frequency_penalty.
+               See: https://docs.litellm.ai/docs/completion/input
+               Note: These params control the optimizer's reasoning model, NOT the prompt evaluation.
+           name: Optional name for the optimizer instance. This will be used when creating optimizations.
         """
         self.model = model
         self.reasoning_model = model
-        self.model_kwargs = model_kwargs
+        self.model_parameters = model_parameters or {}
         self.verbose = verbose
         self.seed = seed
+        self.name = name
         self._history: list[OptimizationRound] = []
         self.experiment_config = None
         self.llm_call_counter = 0
         self.tool_call_counter = 0
         self._opik_client = None  # Lazy initialization
+        self.current_optimization_id: str | None = None  # Track current optimization
+        self.project_name: str = "Optimization"  # Default project name
 
-        # Initialize shared cache
-        initialize_cache()
-
-    def reset_counters(self) -> None:
+    def _reset_counters(self) -> None:
         """Reset all call counters for a new optimization run."""
         self.llm_call_counter = 0
         self.tool_call_counter = 0
 
-    def increment_llm_counter(self) -> None:
+    def _increment_llm_counter(self) -> None:
         """Increment the LLM call counter."""
         self.llm_call_counter += 1
 
-    def increment_tool_counter(self) -> None:
+    def _increment_tool_counter(self) -> None:
         """Increment the tool call counter."""
         self.tool_call_counter += 1
 
@@ -100,7 +103,7 @@ class BaseOptimizer(ABC):
         Should be called when the optimizer is no longer needed.
         """
         # Reset counters
-        self.reset_counters()
+        self._reset_counters()
 
         # Clear history to free memory
         self._history.clear()
@@ -129,8 +132,12 @@ class BaseOptimizer(ABC):
             self._opik_client = opik.Opik()
         return self._opik_client
 
-    def validate_optimization_inputs(
-        self, prompt: "chat_prompt.ChatPrompt", dataset: "Dataset", metric: Callable
+    def _validate_optimization_inputs(
+        self,
+        prompt: "chat_prompt.ChatPrompt",
+        dataset: "Dataset",
+        metric: Callable,
+        support_content_parts: bool = False,
     ) -> None:
         """
         Validate common optimization inputs.
@@ -154,46 +161,86 @@ class BaseOptimizer(ABC):
                 "Metric must be a function that takes `dataset_item` and `llm_output` as arguments."
             )
 
-    def setup_agent_class(
+        if prompt._has_content_parts() and not support_content_parts:
+            raise ValueError(
+                "Prompt has content parts, which are not supported by this optimizer - You can use the Hierarchical Reflective Optimizer instead."
+            )
+
+    def _setup_agent_class(
         self, prompt: "chat_prompt.ChatPrompt", agent_class: Any = None
     ) -> Any:
-        """
-        Setup agent class for optimization.
+        """Resolve the agent class used for prompt evaluations.
+
+        Ensures custom implementations inherit from :class:`OptimizableAgent` and that
+        the optimizer reference is always available to track metrics.
 
         Args:
-            prompt: The chat prompt
-            agent_class: Optional custom agent class
+            prompt: The chat prompt driving the agent instance.
+            agent_class: Optional custom agent class supplied by the caller.
 
         Returns:
-            The agent class to use
+            The agent class to instantiate for dataset evaluations.
         """
         if agent_class is None:
             return create_litellm_agent_class(prompt, optimizer_ref=self)
-        else:
-            return agent_class
+        if not issubclass(agent_class, OptimizableAgent):
+            raise TypeError(
+                f"agent_class must inherit from OptimizableAgent, got {agent_class.__name__}"
+            )
+        return agent_class
 
-    def configure_prompt_model(self, prompt: "chat_prompt.ChatPrompt") -> None:
+    def _bind_optimizer_to_agent(self, agent: OptimizableAgent) -> OptimizableAgent:
+        """Attach this optimizer to the agent instance without mutating the class."""
+        try:
+            agent.optimizer = self  # type: ignore[attr-defined]
+        except Exception:  # pragma: no cover - custom agents may forbid new attrs
+            logger.debug(
+                "Unable to record optimizer on agent instance of %s",
+                agent.__class__.__name__,
+            )
+        return agent
+
+    def _instantiate_agent(
+        self,
+        *args: Any,
+        agent_class: type[OptimizableAgent] | None = None,
+        **kwargs: Any,
+    ) -> OptimizableAgent:
         """
-        Configure prompt model and model_kwargs if not set.
+        Instantiate the provided agent_class (or self.agent_class) and bind optimizer.
+        """
+        resolved_class = agent_class or getattr(self, "agent_class", None)
+        if resolved_class is None:
+            raise ValueError("agent_class must be provided before instantiation")
+        agent = resolved_class(*args, **kwargs)
+        return self._bind_optimizer_to_agent(agent)
+
+    def _extract_tool_prompts(
+        self, tools: list[dict[str, Any]] | None
+    ) -> dict[str, str] | None:
+        """
+        Extract tool names and descriptions from tools list.
 
         Args:
-            prompt: The chat prompt to configure
+            tools: List of tool definitions in OpenAI/LiteLLM format
+
+        Returns:
+            Dictionary mapping tool names to descriptions, or None if no tools
         """
-        # Only configure if prompt is a valid ChatPrompt object
-        if hasattr(prompt, "model") and hasattr(prompt, "model_kwargs"):
-            if prompt.model is None:
-                prompt.model = self.model
-            if prompt.model_kwargs is None:
-                prompt.model_kwargs = self.model_kwargs
+        if not tools:
+            return None
+
+        return {
+            (tool.get("function", {}).get("name") or f"tool_{idx}"): tool.get(
+                "function", {}
+            ).get("description", "")
+            for idx, tool in enumerate(tools)
+        }
 
     # ------------------------------------------------------------------
     # Experiment metadata helpers
     # ------------------------------------------------------------------
 
-    @staticmethod
-    def _drop_none(metadata: dict[str, Any]) -> dict[str, Any]:
-        return {k: v for k, v in metadata.items() if v is not None}
-
     @staticmethod
     def _deep_merge_dicts(
         base: dict[str, Any], overrides: dict[str, Any]
@@ -248,7 +295,7 @@ class BaseOptimizer(ABC):
             params: list[dict[str, Any]] = []
             for parameter in sig.parameters.values():
                 params.append(
-                    self._drop_none(
+                    helpers.drop_none(
                         {
                             "name": parameter.name,
                             "kind": parameter.kind.name,
@@ -265,7 +312,7 @@ class BaseOptimizer(ABC):
                 )
 
             signatures.append(
-                self._drop_none(
+                helpers.drop_none(
                     {
                         "name": name,
                         "parameters": params,
@@ -280,7 +327,8 @@ class BaseOptimizer(ABC):
         agent_config["project_name"] = getattr(prompt, "project_name", None)
         agent_config["model"] = getattr(prompt, "model", None) or self.model
         agent_config["tools"] = self._serialize_tools(prompt)
-        return self._drop_none(agent_config)
+        agent_config["optimizer"] = self.__class__.__name__
+        return helpers.drop_none(agent_config)
 
     def get_optimizer_metadata(self) -> dict[str, Any]:
         """Override in subclasses to expose optimizer-specific parameters."""
@@ -291,7 +339,7 @@ class BaseOptimizer(ABC):
             "name": self.__class__.__name__,
             "version": _OPTIMIZER_VERSION,
             "model": self.model,
-            "model_kwargs": self.model_kwargs or None,
+            "model_parameters": self.model_parameters or None,
             "seed": getattr(self, "seed", None),
             "num_threads": getattr(self, "num_threads", None),
         }
@@ -307,7 +355,36 @@ class BaseOptimizer(ABC):
         if extra_parameters:
             metadata["parameters"] = extra_parameters
 
-        return self._drop_none(metadata)
+        return helpers.drop_none(metadata)
+
+    def _build_optimization_metadata(
+        self, agent_class: type[OptimizableAgent] | None = None
+    ) -> dict[str, Any]:
+        """
+        Build metadata dictionary for optimization creation to be used when
+        creating Opik optimizations.
+
+        Args:
+            agent_class: Optional agent class. If None, will try to get from self.agent_class.
+
+        Returns:
+            Dictionary with 'optimizer' and optionally 'agent_class' keys.
+        """
+        metadata: dict[str, Any] = {"optimizer": self.__class__.__name__}
+        if self.name:
+            metadata["name"] = self.name
+
+        # Try to get agent_class name from parameter or instance
+        agent_class_name: str | None = None
+        if agent_class is not None:
+            agent_class_name = getattr(agent_class, "__name__", None)
+        elif hasattr(self, "agent_class") and self.agent_class is not None:
+            agent_class_name = getattr(self.agent_class, "__name__", None)
+
+        if agent_class_name:
+            metadata["agent_class"] = agent_class_name
+
+        return metadata
 
     def _prepare_experiment_config(
         self,
@@ -325,6 +402,9 @@ class BaseOptimizer(ABC):
             if hasattr(self, "agent_class")
             else None
         )
+        # Ensure experiment traces follow the project name supplied to optimize_prompt.
+        if not project_name:
+            project_name = getattr(self, "project_name", None)
         if not project_name:
             project_name = getattr(prompt, "project_name", None)
         if not project_name:
@@ -341,6 +421,7 @@ class BaseOptimizer(ABC):
             "metric": getattr(metric, "__name__", str(metric)),
             "dataset": getattr(dataset, "name", None),
             "dataset_id": dataset_id,
+            "optimizer": self.__class__.__name__,
             "optimizer_metadata": self._build_optimizer_metadata(),
             "tool_signatures": self._summarize_tool_signatures(prompt),
             "configuration": {
@@ -362,36 +443,7 @@ class BaseOptimizer(ABC):
         if experiment_config:
             base_config = self._deep_merge_dicts(base_config, experiment_config)
 
-        return self._drop_none(base_config)
-
-    def create_optimization_context(
-        self, dataset: "Dataset", metric: Callable, metadata: dict | None = None
-    ) -> Any:
-        """
-        Create optimization context for tracking.
-
-        Args:
-            dataset: The dataset being optimized
-            metric: The metric function
-            metadata: Additional metadata
-
-        Returns:
-            Optimization context manager
-        """
-        context_metadata = {
-            "optimizer": self.__class__.__name__,
-            "model": self.model,
-            "seed": self.seed,
-        }
-        if metadata:
-            context_metadata.update(metadata)
-
-        return optimization_context(
-            client=self.opik_client,
-            dataset_name=dataset.name,
-            objective_name=metric.__name__,
-            metadata=context_metadata,
-        )
+        return helpers.drop_none(base_config)
 
     @abstractmethod
     def optimize_prompt(
@@ -403,6 +455,9 @@ class BaseOptimizer(ABC):
         n_samples: int | None = None,
         auto_continue: bool = False,
         agent_class: type[OptimizableAgent] | None = None,
+        project_name: str = "Optimization",
+        optimization_id: str | None = None,
+        *args: Any,
         **kwargs: Any,
     ) -> optimization_result.OptimizationResult:
         """
@@ -416,60 +471,13 @@ class BaseOptimizer(ABC):
            input_key: input field of dataset
            output_key: output field of dataset
            experiment_config: Optional configuration for the experiment
+           project_name: Opik project name for logging traces (default: "Optimization")
+           optimization_id: Optional ID to use when creating the Opik optimization run;
+               when provided it must be a valid UUIDv7 string.
            **kwargs: Additional arguments for optimization
         """
         pass
 
-    def optimize_mcp(
-        self,
-        prompt: "chat_prompt.ChatPrompt",
-        dataset: Dataset,
-        metric: Callable,
-        *,
-        tool_name: str,
-        second_pass: Any,
-        experiment_config: dict | None = None,
-        n_samples: int | None = None,
-        auto_continue: bool = False,
-        agent_class: type[OptimizableAgent] | None = None,
-        fallback_invoker: Callable[[dict[str, Any]], str] | None = None,
-        fallback_arguments: Callable[[Any], dict[str, Any]] | None = None,
-        allow_tool_use_on_second_pass: bool = False,
-        **kwargs: Any,
-    ) -> optimization_result.OptimizationResult:
-        """
-        Optimize prompts that rely on MCP (Model Context Protocol) tooling.
-
-        This method provides a standardized interface for optimizing prompts that use
-        external tools through the MCP protocol. It handles tool invocation, second-pass
-        coordination, and fallback mechanisms.
-
-        Args:
-            prompt: The chat prompt to optimize, must include tools
-            dataset: Opik dataset containing evaluation data
-            metric: Evaluation function that takes (dataset_item, llm_output) and returns a score
-            tool_name: Name of the MCP tool to use for optimization
-            second_pass: MCPSecondPassCoordinator for handling second-pass tool calls
-            experiment_config: Optional configuration for the experiment
-            n_samples: Number of samples to use for optimization (default: None)
-            auto_continue: Whether to auto-continue optimization (default: False)
-            agent_class: Custom agent class to use (default: None)
-            fallback_invoker: Fallback function for tool invocation (default: None)
-            fallback_arguments: Function to extract tool arguments (default: None)
-            allow_tool_use_on_second_pass: Whether to allow tool use on second pass (default: False)
-            **kwargs: Additional arguments for optimization
-
-        Returns:
-            OptimizationResult: The optimization result containing the optimized prompt and metrics
-
-        Raises:
-            NotImplementedError: If the optimizer doesn't implement MCP optimization
-            ValueError: If the prompt doesn't include required tools
-        """
-        raise NotImplementedError(
-            f"{self.__class__.__name__} does not implement optimize_mcp yet."
-        )
-
     def get_history(self) -> list[OptimizationRound]:
         """
         Get the optimization history.
@@ -488,7 +496,7 @@ class BaseOptimizer(ABC):
         """
         self._history.append(round_data)
 
-    def update_optimization(
+    def _update_optimization(
         self, optimization: optimization.Optimization, status: str
     ) -> None:
         """
@@ -521,11 +529,6 @@ class BaseOptimizer(ABC):
     ) -> float:
         random.seed(seed)
 
-        if prompt.model is None:
-            prompt.model = self.model
-        if prompt.model_kwargs is None:
-            prompt.model_kwargs = self.model_kwargs
-
         self.agent_class: type[OptimizableAgent]
 
         if agent_class is None:
@@ -533,14 +536,21 @@ class BaseOptimizer(ABC):
         else:
             self.agent_class = agent_class
 
-        agent = self.agent_class(prompt)
+        agent = self._instantiate_agent(prompt)
 
         def llm_task(dataset_item: dict[str, Any]) -> dict[str, str]:
             messages = prompt.get_messages(dataset_item)
             raw_model_output = agent.invoke(messages)
             cleaned_model_output = raw_model_output.strip()
+
+            # Add tags to trace for optimization tracking
+            if self.current_optimization_id:
+                opik_context.update_current_trace(
+                    tags=[self.current_optimization_id, "Evaluation"]
+                )
+
             result = {
-                mappers.EVALUATED_LLM_TASK_OUTPUT: cleaned_model_output,
+                "llm_output": cleaned_model_output,
             }
             return result
 
@@ -556,6 +566,7 @@ class BaseOptimizer(ABC):
                 raise Exception("Can't use n_samples and dataset_item_ids")
 
             all_ids = [dataset_item["id"] for dataset_item in dataset.get_items()]
+            n_samples = min(n_samples, len(all_ids))
             dataset_item_ids = random.sample(all_ids, n_samples)
 
         score = task_evaluator.evaluate(
@@ -564,9 +575,9 @@ class BaseOptimizer(ABC):
             metric=metric,
             evaluated_task=llm_task,
             num_threads=n_threads,
-            project_name=experiment_config.get("project_name"),
+            project_name=self.project_name,
             experiment_config=experiment_config,
-            optimization_id=None,
+            optimization_id=self.current_optimization_id,
             verbose=verbose,
         )
         return score
