@@ -3,13 +3,16 @@
 from .core import *  # noqa: F401,F403
 from .dataset_utils import *  # noqa: F401,F403
 from .prompt_segments import *  # noqa: F401,F403
+from .candidate_utils import *  # noqa: F401,F403
 
 from . import core as _core
 from . import dataset_utils as _dataset_utils
 from . import prompt_segments as _prompt_segments
+from . import candidate_utils as _candidate_utils
 
 __all__: list[str] = [
     *getattr(_core, "__all__", []),
     *getattr(_dataset_utils, "__all__", []),
     *getattr(_prompt_segments, "__all__", []),
+    *getattr(_candidate_utils, "__all__", []),
 ]
